clear ;

%% Read in data.  CSHomePrice - panel norm; circuit specific trends
names1 = importfile('temp_CSHomePrice.mat.csv');
% NOTE: importfile cannot handle text data
cases = 'logCSHPI, Panel';

% Read in additional probability controls 
prnames = importfile('probs.txt');

% Match new probability controls up by circuit and year
for kk = 3:size(prnames,1),
    assignin('base',strcat('F_',prnames{kk}),zeros(size(circuit)));
end
for ii = 1:max(circuit),
    for jj = min(year):max(year),
        I = find(Scircuit == ii & Syear == jj); %#ok<NASGU>
        J = find(circuit == ii & year == jj); %#ok<NASGU>
        for kk = 3:size(prnames,1),
            eval(strcat('F_',prnames{kk},'(J) = ',prnames{kk},'(I);'));
        end
    end
end
        

% Recode year and circuit dummy variables and prevent rank deficiency
yearn = nominal(year);
iyearn = dummyvar(yearn);
circuitn = nominal(circuit);
icircuitn = dummyvar(circuitn);
icircuitn(:,1) = [];
iyearn(:,1) = [];
col1 = ones(size(iyearn,1),1);
jcircuityear = zeros(size(icircuitn));
for i = 1:size(icircuitn, 2),
    jcircuityear(:,i) = year.*icircuitn(:,i);
end

cy = nominal((recode(circuit)-1).*max(recode(year))+recode(year));
Dt = dummyvar(cy);

% Endogenous, d
d = numpro_casecat_12;
% Exogenous to be partialed out, x
x = [missing_cy_12, numcasecat_12, icircuitn, iyearn, jcircuityear, col1];
% Candidate instruments, z:
z = 1*[numpanels1x_noreligion, numpanels1x_jd_public, ...
    numpanels1x_dem, numpanels2x_dem, numpanels3x_dem, ...
    numpanels1x_dem.^2 , numpanels1x_dem.^3 , ...
    numpanels1x_female, numpanels2x_female, numpanels3x_female, ...
    numpanels1x_nonwhite, numpanels2x_nonwhite, numpanels3x_nonwhite, ...
    numpanels1x_black, numpanels2x_black, numpanels3x_black, ...
    numpanels1x_jewish, numpanels2x_jewish, numpanels3x_jewish, ...
    numpanels1x_catholic, numpanels2x_catholic, numpanels3x_catholic, ...
    numpanels2x_noreligion, numpanels3x_noreligion, ...
    numpanels1x_instate_ba, numpanels2x_instate_ba, numpanels3x_instate_ba, ...
    numpanels1x_ba_public, numpanels2x_ba_public, numpanels3x_ba_public, ...
    numpanels2x_jd_public, numpanels3x_jd_public, ...
    (numpanels1x_jd_public.^2)/5 , (numpanels1x_jd_public.^3)/25 , ...
    numpanels1x_elev, numpanels2x_elev, numpanels3x_elev, ...
    (numpanels1x_elev.^2) , (numpanels1x_elev.^3) , ...
    numpanels1x_female_black, numpanels2x_female_black, numpanels3x_female_black, ...
    numpanels1x_female_noreligion, numpanels2x_female_noreligion, numpanels3x_female_noreligion, ...
    numpanels1x_black_noreligion, numpanels2x_black_noreligion, numpanels3x_black_noreligion, ...
    numpanels1x_female_jd_public, numpanels2x_female_jd_public, numpanels3x_female_jd_public, ...
    numpanels1x_black_jd_public, numpanels2x_black_jd_public, numpanels3x_black_jd_public, ...
    numpanels1x_noreligion_jd_pub, numpanels2x_noreligion_jd_pub, numpanels3x_noreligion_jd_pub, ...
    numpanels1x_mainline, numpanels2x_mainline, numpanels3x_mainline, ...
    numpanels1x_evangelical, numpanels2x_evangelical, numpanels3x_evangelical, ...
    numpanels1x_protestant, numpanels2x_protestant, numpanels3x_protestant, ...
    numpanels1x_dem.*numpanels1x_female , numpanels1x_dem.*numpanels1x_black , ...
    numpanels1x_dem.*numpanels1x_jewish , numpanels1x_dem.*numpanels1x_catholic , ...
    numpanels1x_dem.*numpanels1x_noreligion , numpanels1x_dem.*numpanels1x_instate_ba , ...
    numpanels1x_dem.*numpanels1x_jd_public , numpanels1x_dem.*numpanels1x_mainline , ...
    numpanels1x_dem.*numpanels1x_evangelical , numpanels1x_dem.*numpanels1x_protestant , ...
    numpanels1x_dem.*numpanels1x_ba_public , numpanels1x_dem.*numpanels1x_elev , ...
    numpanels1x_dem.*numpanels1x_nonwhite , ...
    numpanels1x_female.*numpanels1x_jewish , numpanels1x_female.*numpanels1x_catholic , ...
    numpanels1x_female.*numpanels1x_noreligion , numpanels1x_female.*numpanels1x_instate_ba , ...
    numpanels1x_female.*numpanels1x_jd_public , numpanels1x_female.*numpanels1x_mainline , ...
    numpanels1x_female.*numpanels1x_evangelical , numpanels1x_female.*numpanels1x_protestant , ...
    numpanels1x_female.*numpanels1x_ba_public , numpanels1x_female.*numpanels1x_elev , ...
    numpanels1x_female.*numpanels1x_nonwhite , ...
    numpanels1x_jd_public.*numpanels1x_jewish , numpanels1x_jd_public.*numpanels1x_catholic , ...
    numpanels1x_jd_public.*numpanels1x_noreligion , numpanels1x_jd_public.*numpanels1x_instate_ba , ...
    numpanels1x_jd_public.*numpanels1x_mainline , ...
    numpanels1x_jd_public.*numpanels1x_evangelical , numpanels1x_jd_public.*numpanels1x_protestant , ...
    numpanels1x_jd_public.*numpanels1x_ba_public , numpanels1x_jd_public.*numpanels1x_elev , ...
    numpanels1x_jd_public.*numpanels1x_nonwhite , ...
    numpanels1x_ba_public.*numpanels1x_jewish , numpanels1x_ba_public.*numpanels1x_catholic , ...
    numpanels1x_ba_public.*numpanels1x_noreligion , numpanels1x_ba_public.*numpanels1x_instate_ba , ...
    numpanels1x_ba_public.*numpanels1x_mainline , ...
    numpanels1x_ba_public.*numpanels1x_evangelical , numpanels1x_ba_public.*numpanels1x_protestant , ...
    numpanels1x_ba_public.*numpanels1x_elev , ...
    numpanels1x_ba_public.*numpanels1x_nonwhite , ...
    numpanels2x_jd_public.*numpanels1x_jewish , numpanels2x_jd_public.*numpanels1x_catholic , ...
    numpanels2x_jd_public.*numpanels1x_noreligion , numpanels2x_jd_public.*numpanels1x_instate_ba , ...
    numpanels2x_jd_public.*numpanels1x_mainline , ...
    numpanels2x_jd_public.*numpanels1x_evangelical , numpanels2x_jd_public.*numpanels1x_protestant , ...
    numpanels2x_jd_public.*numpanels1x_ba_public , numpanels2x_jd_public.*numpanels1x_elev , ...
    numpanels2x_jd_public.*numpanels1x_nonwhite , ...
    numpanels3x_jd_public.*numpanels1x_jewish , numpanels3x_jd_public.*numpanels1x_catholic , ...
    numpanels3x_jd_public.*numpanels1x_noreligion , numpanels3x_jd_public.*numpanels1x_instate_ba , ...
    numpanels3x_jd_public.*numpanels1x_mainline , ...
    numpanels3x_jd_public.*numpanels1x_evangelical , numpanels3x_jd_public.*numpanels1x_protestant , ...
    numpanels3x_jd_public.*numpanels1x_ba_public , numpanels3x_jd_public.*numpanels1x_elev , ...
    numpanels3x_jd_public.*numpanels1x_nonwhite , ...
    numpanels2x_female.*numpanels1x_jewish , numpanels2x_female.*numpanels1x_catholic , ...
    numpanels2x_female.*numpanels1x_noreligion , numpanels2x_female.*numpanels1x_instate_ba , ...
    numpanels2x_female.*numpanels1x_jd_public , numpanels2x_female.*numpanels1x_mainline , ...
    numpanels2x_female.*numpanels1x_evangelical , numpanels2x_female.*numpanels1x_protestant , ...
    numpanels2x_female.*numpanels1x_ba_public , numpanels2x_female.*numpanels1x_elev , ...
    numpanels2x_female.*numpanels1x_nonwhite , ...
    numpanels2x_ba_public.*numpanels1x_jewish , numpanels2x_ba_public.*numpanels1x_catholic , ...
    numpanels2x_ba_public.*numpanels1x_noreligion , numpanels2x_ba_public.*numpanels1x_instate_ba , ...
    numpanels2x_ba_public.*numpanels1x_mainline , ...
    numpanels2x_ba_public.*numpanels1x_evangelical , numpanels2x_ba_public.*numpanels1x_protestant , ...
    numpanels2x_ba_public.*numpanels1x_elev , ...
    numpanels2x_ba_public.*numpanels1x_nonwhite , ...
    numpanels1x_instate_ba.*numpanels1x_jewish , numpanels1x_instate_ba.*numpanels1x_catholic , ...
    numpanels1x_instate_ba.*numpanels1x_noreligion , numpanels1x_instate_ba.*numpanels1x_instate_ba , ...
    numpanels1x_instate_ba.*numpanels1x_mainline , ...
    numpanels1x_instate_ba.*numpanels1x_evangelical , numpanels1x_instate_ba.*numpanels1x_protestant , ...
    numpanels1x_instate_ba.*numpanels1x_elev , ...
    numpanels1x_instate_ba.*numpanels1x_nonwhite , ...
    numpanels1x_elev.*numpanels1x_jewish , numpanels1x_elev.*numpanels1x_catholic , ...
    numpanels1x_elev.*numpanels1x_noreligion , ...
    numpanels1x_elev.*numpanels1x_mainline , ...
    numpanels1x_elev.*numpanels1x_evangelical , numpanels1x_elev.*numpanels1x_protestant , ...
    numpanels1x_elev.*numpanels1x_nonwhite , ...
    numpanels1x_protestant.*numpanels1x_jewish , numpanels1x_protestant.*numpanels1x_catholic , ...
    numpanels1x_protestant.*numpanels1x_noreligion , ...
    numpanels1x_protestant.*numpanels1x_mainline , ...
    numpanels1x_protestant.*numpanels1x_evangelical , numpanels1x_protestant.*numpanels1x_protestant , ...
    numpanels1x_protestant.*numpanels1x_nonwhite , ...
    numpanels1x_mainline.*numpanels1x_jewish , numpanels1x_mainline.*numpanels1x_catholic , ...
    numpanels1x_mainline.*numpanels1x_noreligion , numpanels1x_mainline.*numpanels1x_evangelical , ...
    numpanels1x_mainline.*numpanels1x_mainline , ...
    numpanels1x_mainline.*numpanels1x_nonwhite ];
% Candidate instruments' names, namez
namez = {'1x_noreligion', '1x_jd_public', '1x_dem', '2x_dem', '3x_dem', ...
    '1x_dem^2', '1x_dem^3', ...
    '1x_female', '2x_female', '3x_female', ...
    '1x_nonwhite', '2x_nonwhite', '3x_nonwhite', ...
    '1x_black', '2x_black', '3x_black', ...
    '1x_jewish', '2x_jewish', '3x_jewish', ...
    '1x_catholic', '2x_catholic', '3x_catholic', ...
    '2x_noreligion', '3x_noreligion', ...
    '1x_instate_ba', '2x_instate_ba', '3x_instate_ba', ...
    '1x_ba_public', '2x_ba_public', '3x_ba_public', ...
    '2x_jd_public', '3x_jd_public', ...
    '1x_jd_public^2', '1x_jd_public^3' , ...
    '1x_elev', '2x_elev', '3x_elev', ...
    '1x_elev^2' , '1x_elev^3' , ...
    '1x_female_black', '2x_female_black', '3x_female_black', ...
    '1x_female_noreligion', '2x_female_noreligion', '3x_female_noreligion', ...
    '1x_black_noreligion', '2x_black_noreligion', '3x_black_noreligion', ...
    '1x_female_jd_public', '2x_female_jd_public', '3x_female_jd_public', ...
    '1x_black_jd_public', '2x_black_jd_public', '3x_black_jd_public', ...
    '1x_noreligion_jd_pub', '2x_noreligion_jd_pub', '3x_noreligion_jd_pub', ...
    '1x_mainline', '2x_mainline', '3x_mainline', ...
    '1x_evangelical', '2x_evangelical', '3x_evangelical', ...
    '1x_protestant', '2x_protestant', '3x_protestant', ...
    '1x_dem.*1x_female' , '1x_dem.*1x_black' , ...
    '1x_dem.*1x_jewish' , '1x_dem.*1x_catholic' , ...
    '1x_dem.*1x_noreligion' , '1x_dem.*1x_ba_public' , ...
    '1x_dem.*1x_jd_public' , '1x_dem.*1x_mainline' , ...
    '1x_dem.*1x_evangelical' , '1x_dem.*1x_protestant' , ...
    '1x_dem.*1x_ba_public', '1x_dem.*1x_elev' , '1x_dem.*1x_nonwhite' , ... 
    '1x_female.*1x_jewish' , '1x_female.*1x_catholic' , ...
    '1x_female.*1x_noreligion' , '1x_female.*1x_ba_public' , ...
    '1x_female.*1x_jd_public' , '1x_female.*1x_mainline' , ...
    '1x_female.*1x_evangelical' , '1x_female.*1x_protestant' , ...
    '1x_female.*1x_ba_public', '1x_female.*1x_elev' , '1x_female.*1x_nonwhite' , ... 
    '1x_jd_public.*1x_jewish' , '1x_jd_public.*1x_catholic' , ...
    '1x_jd_public.*1x_noreligion' , '1x_jd_public.*1x_instate_ba' , ...
    '1x_jd_public.*1x_mainline' , ...
    '1x_jd_public.*1x_evangelical' , '1x_jd_public.*1x_protestant' , ...
    '1x_jd_public.*1x_ba_public', '1x_jd_public.*1x_elev' , ...
    '1x_jd_public.*1x_nonwhite' , ...
    '1x_ba_public.*1x_jewish' , '1x_ba_public.*1x_catholic' , ...
    '1x_ba_public.*1x_noreligion' , '1x_ba_public.*1x_instate_ba', ...
    '1x_ba_public.*1x_mainline' , ...
    '1x_ba_public.*1x_evangelical' , '1x_ba_public.*1x_protestant' , ...
    '1x_ba_public.*1x_elev' , ...
    '1x_ba_public.*1x_nonwhite' , ...
    '2x_jd_public.*1x_jewish' , '2x_jd_public.*1x_catholic' , ...
    '2x_jd_public.*1x_noreligion' , '2x_jd_public.*1x_instate_ba' , ...
    '2x_jd_public.*1x_mainline' , ...
    '2x_jd_public.*1x_evangelical' , '2x_jd_public.*1x_protestant' , ...
    '2x_jd_public.*1x_ba_public', '2x_jd_public.*1x_elev' , ...
    '2x_jd_public.*1x_nonwhite' , ...
    '3x_jd_public.*1x_jewish' , '3x_jd_public.*1x_catholic' , ...
    '3x_jd_public.*1x_noreligion' , '3x_jd_public.*1x_instate_ba' , ...
    '3x_jd_public.*1x_mainline' , ...
    '3x_jd_public.*1x_evangelical' , '3x_jd_public.*1x_protestant' , ...
    '3x_jd_public.*1x_ba_public', '3x_jd_public.*1x_elev' , ...
    '3x_jd_public.*1x_nonwhite' , ...
    '2x_female.*1x_jewish' , '2x_female.*1x_catholic' , ...
    '2x_female.*1x_noreligion' , '2x_female.*1x_ba_public' , ...
    '2x_female.*1x_jd_public' , '2x_female.*1x_mainline' , ...
    '2x_female.*1x_evangelical' , '2x_female.*1x_protestant' , ...
    '2x_female.*1x_ba_public', '2x_female.*1x_elev' , '2x_female.*1x_nonwhite' , ... 
    '2x_ba_public.*1x_jewish' , '2x_ba_public.*1x_catholic' , ...
    '2x_ba_public.*1x_noreligion' , '2x_ba_public.*1x_instate_ba', ...
    '2x_ba_public.*1x_mainline' , ...
    '2x_ba_public.*1x_evangelical' , '2x_ba_public.*1x_protestant' , ...
    '2x_ba_public.*1x_elev' , ...
    '2x_ba_public.*1x_nonwhite' , ...
    '1x_instate_ba.*1x_jewish' , '1x_instate_ba.*1x_catholic' , ...
    '1x_instate_ba.*1x_noreligion' , '1x_instate_ba.*1x_instate_ba', ...
    '1x_instate_ba.*1x_mainline' , ...
    '1x_instate_ba.*1x_evangelical' , '1x_instate_ba.*1x_protestant' , ...
    '1x_instate_ba.*1x_elev' , ...
    '1x_instate_ba.*1x_nonwhite' , ...    
    '1x_elev.*1x_jewish' , '1x_elev.*1x_catholic' , ...
    '1x_elev.*1x_noreligion' , '1x_elev.*1x_mainline' , ...
    '1x_elev.*1x_evangelical' , '1x_elev.*1x_protestant' , ...
    '1x_elev.*1x_nonwhite' , ...
    '1x_protestant.*1x_jewish' , '1x_protestant.*1x_catholic' , ...
    '1x_protestant.*1x_noreligion' , '1x_protestant.*1x_mainline' , ...
    '1x_protestant.*1x_evangelical' , '1x_protestant.*1x_protestant' , ...
    '1x_protestant.*1x_nonwhite' , ...
    '1x_mainline.*1x_jewish' , '1x_mainline.*1x_catholic' , ...
    '1x_mainline.*1x_noreligion' , ...
    '1x_mainline.*1x_evangelical' , '1x_mainline.*1x_mainline' , ...
    '1x_mainline.*1x_nonwhite' }';

% Probability controls, xP:
xP = 1*[F_prob_1x_dem, F_prob_2x_dem, F_prob_3x_dem, ...
    F_prob_1x_female, F_prob_2x_female, F_prob_3x_female, ...
    F_prob_1x_nonwhite, F_prob_2x_nonwhite, F_prob_3x_nonwhite, ...
    F_prob_1x_black, F_prob_2x_black, F_prob_3x_black, ...
    F_prob_1x_jewish, F_prob_2x_jewish, F_prob_3x_jewish, ...
    F_prob_1x_catholic, F_prob_2x_catholic, F_prob_3x_catholic, ...
    F_prob_1x_noreligion, F_prob_2x_noreligion, F_prob_3x_noreligion, ...
    F_prob_1x_instate_ba, F_prob_2x_instate_ba, F_prob_3x_instate_ba, ...
    F_prob_1x_ba_public, F_prob_2x_ba_public, F_prob_3x_ba_public, ...
    F_prob_1x_jd_public, F_prob_2x_jd_public, F_prob_3x_jd_public, ...
    F_prob_1x_elev, F_prob_2x_elev, F_prob_3x_elev, ...
    F_prob_1x_female_black, F_prob_2x_female_black, F_prob_3x_female_black, ...
    F_prob_1x_female_noreligion, F_prob_2x_female_noreligion, F_prob_3x_female_noreligion, ...
    F_prob_1x_black_noreligion, F_prob_2x_black_noreligion, F_prob_3x_black_noreligion, ...
    F_prob_1x_female_jd_public, F_prob_2x_female_jd_public, F_prob_3x_female_jd_public, ...
    F_prob_1x_black_jd_public, F_prob_2x_black_jd_public, F_prob_3x_black_jd_public, ...
    F_prob_1x_noreligion_jd_public, F_prob_2x_noreligion_jd_public, F_prob_3x_noreligion_jd_public, ...
    F_prob_1x_mainline, F_prob_2x_mainline, F_prob_3x_mainline, ...
    F_prob_1x_evangelical, F_prob_2x_evangelical, F_prob_3x_evangelical, ...
    F_prob_1x_protestant, F_prob_2x_protestant, F_prob_3x_protestant ]; 

% # Democrats instrument
IND_dem = zeros(size(z,2),1);
IND_dem(3) = 1;
IND_dem = logical(IND_dem);


%% LASSO runs with full set of probability controls
% Partial out x's that need to show up in both stages
I = find(mean(z) >= .05);
z = z(:,I);
IND_dem = IND_dem(I);
namez = namez(I);
y = logpriceindex;
I = find(mean(xP) >= .05);
xP = xP(:,I); %#ok<FNDSB>
x = [x xP];

% Collapse to circuit-year level
x = inv(Dt'*Dt)*(Dt'*x);
y = inv(Dt'*Dt)*(Dt'*y);
d = inv(Dt'*Dt)*(Dt'*d);
z = inv(Dt'*Dt)*(Dt'*z);

xxinv = inv(x'*x);
My = full(y - x*xxinv*(x'*y)); %#ok<*MINV>
Md = full(d - x*xxinv*(x'*d));
Mz = full(z - x*xxinv*(x'*z));

I = find(std(Mz) > 1e-6);
Mz = Mz(:,I);
IND_dem = IND_dem(I);
namez = namez(I);

[I,J] = find(abs(tril(corr(Mz)-eye(size(Mz,2)))) > .99);
drop = unique(I);
Mz(:,drop) = [];
IND_dem(drop) = [];
namez(drop) = [];

n = size(Mz,1);
p = size(Mz,2);

%% Rule-of-thumb
c = 1.1; % LASSO penalty multiplier
gamma = .1/log(max(p,n));
lambda = c*2*sqrt(n)*norminv(1-gamma/(2*p));
% lambda = LassoSimulateLambda( Mz, 1, 10000, n, gamma );

piRoTI = feasiblePostLasso(Md,Mz,'lambda',lambda);
sum(abs(piRoTI) > 0);
indRoTI = abs(piRoTI) > 0;


%% 10-Fold Cross-validation
rng(13100705);
group = rand(n,1);
groupprc = [0,prctile(group,10:10:90),1];
nC = 10;

lambdaGrid = (70:2:130)';
nG = size(lambdaGrid,1);
CVI = zeros(nG,nC);
for ii = 1:nG
    disp(lambdaGrid(ii));
    for jj = 1:nC
        pred = (group > groupprc(jj) & group <= groupprc(jj+1));
        est = logical(1-pred);
        xjj = x(est,:);
        Mdjj = d(est,:) - xjj*(xjj\d(est,:));
        Mzjj = z(est,:) - xjj*(xjj\z(est,:));
        Ijj = find(std(Mzjj) > 1e-6);
        Mzjj = Mzjj(:,Ijj);
        zjj = z(:,Ijj);
        [Ijj,Jjj] = find(abs(tril(corr(Mzjj)-eye(size(Mzjj,2)))) > .99);
        drop = unique(Ijj);
        Mzjj(:,drop) = [];
        zjj(:,drop) = [];
        piIjj = feasiblePostLasso(Mdjj,Mzjj,...
            'lambda',lambdaGrid(ii));
        indIjj = abs(piIjj) > 0;
        dHatIjj = [x(pred,:) z(pred,indIjj)]*([x(est,:) z(est,indIjj)]\d(est,:));
        CVI(ii,jj) = sum((d(pred,:)-dHatIjj).^2);
    end
end

meanCVI = mean(CVI,2);

stdCVI = std(CVI,[],2);

c68 = tinv(1-.16,9);

mCVy = meanCVI == min(meanCVI);

by = max(meanCVI(mCVy) + c68*stdCVI(mCVy));

lambdaCVI = sqrt(10/9)*max(lambdaGrid(meanCVI < by));
piCVI = feasiblePostLasso(Md,Mz,'lambda',lambdaCVI);
sum(abs(piCVI) > 0)
indCVI = abs(piCVI) > 0;

%% Obtain FS and 2SLS estimates

% Number of nonredundant partialed out x's
kx = size(x,2);

% OLS Estimates
OLS = Md\My;
et = My-Md*OLS;
sOLS = sqrt((n-1)/(n-kx-1))*hetero_se(Md,et,inv(Md'*Md));

% (1) Calculate results using LASSO rule-of-thumb - independent
if sum(indRoTI) > 0
    % (A) first stage
    zt = Mz(:,indRoTI);
    FSRoTI = inv(zt'*zt)*(zt'*Md);
    ut = Md-zt*FSRoTI;
    sFSRoTI = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*hetero_se(zt,ut,inv(zt'*zt));
    
    % (B) Calculate 2sls estimate
    TSLSRoTI = inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)*(zt'*My));
    et = My-Md*TSLSRoTI;
    sTSLSRoTI = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
        inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));
end

%% (2) Calculate results using LASSO CV - independent
if sum(indCVI) > 0
    % (A) first stage
    zt = Mz(:,indCVI);
    FSCVI = inv(zt'*zt)*(zt'*Md);
    ut = Md-zt*FSCVI;
    sFSCVI = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*hetero_se(zt,ut,inv(zt'*zt));
    
    % (B) Calculate 2sls estimate
    TSLSCVI = inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)*(zt'*My));
    et = My-Md*TSLSCVI;
    sTSLSCVI = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
        inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));
end


%% Democrat Baseline
% (A) first stage
zt = Mz(:,IND_dem);
FSCVD = inv(zt'*zt)*(zt'*Md);
ut = Md-zt*FSCVD;
sFSCVD = sqrt((n-size(zt,2))\(n-size(zt,2)-kx))*hetero_se(zt,ut,inv(zt'*zt));

% (B) Calculate 2sls estimate
TSLSCVD = inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)*(zt'*My));
et = My-Md*TSLSCVD;
sTSLSCVD = sqrt((n-1)/(n-kx-1))*hetero_se(zt,et,...
    inv((Md'*zt)*inv(zt'*zt)*(zt'*Md))*((Md'*zt)*inv(zt'*zt)));

