clear ;

%% Read in data.  CSHomePrice - panel norm; circuit specific trends
names1 = importfile('temp_CSHomePrice.mat.csv');
% NOTE: importfile cannot handle text data
cases = 'logCSHPI, Panel';

% Read in additional probability controls 
prnames = importfile('probs.txt');

% Match new probability controls up by circuit and year
for kk = 3:size(prnames,1),
    assignin('base',strcat('F_',prnames{kk}),zeros(size(circuit)));
end
for ii = 1:max(circuit),
    for jj = min(year):max(year),
        I = find(Scircuit == ii & Syear == jj); 
        J = find(circuit == ii & year == jj); 
        for kk = 3:size(prnames,1),
            eval(strcat('F_',prnames{kk},'(J) = ',prnames{kk},'(I);'));
        end
    end
end
        

% Recode year and circuit dummy variables and prevent rank deficiency
yearn = nominal(year);
iyearn = dummyvar(yearn);
circuitn = nominal(circuit);
icircuitn = dummyvar(circuitn);
icircuitn(:,1) = [];
iyearn(:,1) = [];
col1 = ones(size(iyearn,1),1);
jcircuityear = zeros(size(icircuitn));
for i = 1:size(icircuitn, 2),
    jcircuityear(:,i) = year.*icircuitn(:,i);
end

cy = nominal((recode(circuit)-1).*max(recode(year))+recode(year));
Dt = dummyvar(cy);

% Endogenous, d
d = numpro_casecat_12;
% Exogenous to be partialed out, x
x = [missing_cy_12, numcasecat_12, icircuitn, iyearn, jcircuityear, col1];
% Candidate instruments, z:
z = 1*[numpanels1x_noreligion, numpanels1x_jd_public, ...
    numpanels1x_dem, numpanels2x_dem, numpanels3x_dem, ...
    numpanels1x_dem.^2 , numpanels1x_dem.^3 , ...
    numpanels1x_female, numpanels2x_female, numpanels3x_female, ...
    numpanels1x_nonwhite, numpanels2x_nonwhite, numpanels3x_nonwhite, ...
    numpanels1x_black, numpanels2x_black, numpanels3x_black, ...
    numpanels1x_jewish, numpanels2x_jewish, numpanels3x_jewish, ...
    numpanels1x_catholic, numpanels2x_catholic, numpanels3x_catholic, ...
    numpanels2x_noreligion, numpanels3x_noreligion, ...
    numpanels1x_instate_ba, numpanels2x_instate_ba, numpanels3x_instate_ba, ...
    numpanels1x_ba_public, numpanels2x_ba_public, numpanels3x_ba_public, ...
    numpanels2x_jd_public, numpanels3x_jd_public, ...
    (numpanels1x_jd_public.^2)/5 , (numpanels1x_jd_public.^3)/25 , ...
    numpanels1x_elev, numpanels2x_elev, numpanels3x_elev, ...
    (numpanels1x_elev.^2) , (numpanels1x_elev.^3) , ...
    numpanels1x_female_black, numpanels2x_female_black, numpanels3x_female_black, ...
    numpanels1x_female_noreligion, numpanels2x_female_noreligion, numpanels3x_female_noreligion, ...
    numpanels1x_black_noreligion, numpanels2x_black_noreligion, numpanels3x_black_noreligion, ...
    numpanels1x_female_jd_public, numpanels2x_female_jd_public, numpanels3x_female_jd_public, ...
    numpanels1x_black_jd_public, numpanels2x_black_jd_public, numpanels3x_black_jd_public, ...
    numpanels1x_noreligion_jd_pub, numpanels2x_noreligion_jd_pub, numpanels3x_noreligion_jd_pub, ...
    numpanels1x_mainline, numpanels2x_mainline, numpanels3x_mainline, ...
    numpanels1x_evangelical, numpanels2x_evangelical, numpanels3x_evangelical, ...
    numpanels1x_protestant, numpanels2x_protestant, numpanels3x_protestant, ...
    numpanels1x_dem.*numpanels1x_female , numpanels1x_dem.*numpanels1x_black , ...
    numpanels1x_dem.*numpanels1x_jewish , numpanels1x_dem.*numpanels1x_catholic , ...
    numpanels1x_dem.*numpanels1x_noreligion , numpanels1x_dem.*numpanels1x_instate_ba , ...
    numpanels1x_dem.*numpanels1x_jd_public , numpanels1x_dem.*numpanels1x_mainline , ...
    numpanels1x_dem.*numpanels1x_evangelical , numpanels1x_dem.*numpanels1x_protestant , ...
    numpanels1x_dem.*numpanels1x_ba_public , numpanels1x_dem.*numpanels1x_elev , ...
    numpanels1x_dem.*numpanels1x_nonwhite , ...
    numpanels1x_female.*numpanels1x_jewish , numpanels1x_female.*numpanels1x_catholic , ...
    numpanels1x_female.*numpanels1x_noreligion , numpanels1x_female.*numpanels1x_instate_ba , ...
    numpanels1x_female.*numpanels1x_jd_public , numpanels1x_female.*numpanels1x_mainline , ...
    numpanels1x_female.*numpanels1x_evangelical , numpanels1x_female.*numpanels1x_protestant , ...
    numpanels1x_female.*numpanels1x_ba_public , numpanels1x_female.*numpanels1x_elev , ...
    numpanels1x_female.*numpanels1x_nonwhite , ...
    numpanels1x_jd_public.*numpanels1x_jewish , numpanels1x_jd_public.*numpanels1x_catholic , ...
    numpanels1x_jd_public.*numpanels1x_noreligion , numpanels1x_jd_public.*numpanels1x_instate_ba , ...
    numpanels1x_jd_public.*numpanels1x_mainline , ...
    numpanels1x_jd_public.*numpanels1x_evangelical , numpanels1x_jd_public.*numpanels1x_protestant , ...
    numpanels1x_jd_public.*numpanels1x_ba_public , numpanels1x_jd_public.*numpanels1x_elev , ...
    numpanels1x_jd_public.*numpanels1x_nonwhite , ...
    numpanels1x_ba_public.*numpanels1x_jewish , numpanels1x_ba_public.*numpanels1x_catholic , ...
    numpanels1x_ba_public.*numpanels1x_noreligion , numpanels1x_ba_public.*numpanels1x_instate_ba , ...
    numpanels1x_ba_public.*numpanels1x_mainline , ...
    numpanels1x_ba_public.*numpanels1x_evangelical , numpanels1x_ba_public.*numpanels1x_protestant , ...
    numpanels1x_ba_public.*numpanels1x_elev , ...
    numpanels1x_ba_public.*numpanels1x_nonwhite , ...
    numpanels2x_jd_public.*numpanels1x_jewish , numpanels2x_jd_public.*numpanels1x_catholic , ...
    numpanels2x_jd_public.*numpanels1x_noreligion , numpanels2x_jd_public.*numpanels1x_instate_ba , ...
    numpanels2x_jd_public.*numpanels1x_mainline , ...
    numpanels2x_jd_public.*numpanels1x_evangelical , numpanels2x_jd_public.*numpanels1x_protestant , ...
    numpanels2x_jd_public.*numpanels1x_ba_public , numpanels2x_jd_public.*numpanels1x_elev , ...
    numpanels2x_jd_public.*numpanels1x_nonwhite , ...
    numpanels3x_jd_public.*numpanels1x_jewish , numpanels3x_jd_public.*numpanels1x_catholic , ...
    numpanels3x_jd_public.*numpanels1x_noreligion , numpanels3x_jd_public.*numpanels1x_instate_ba , ...
    numpanels3x_jd_public.*numpanels1x_mainline , ...
    numpanels3x_jd_public.*numpanels1x_evangelical , numpanels3x_jd_public.*numpanels1x_protestant , ...
    numpanels3x_jd_public.*numpanels1x_ba_public , numpanels3x_jd_public.*numpanels1x_elev , ...
    numpanels3x_jd_public.*numpanels1x_nonwhite , ...
    numpanels2x_female.*numpanels1x_jewish , numpanels2x_female.*numpanels1x_catholic , ...
    numpanels2x_female.*numpanels1x_noreligion , numpanels2x_female.*numpanels1x_instate_ba , ...
    numpanels2x_female.*numpanels1x_jd_public , numpanels2x_female.*numpanels1x_mainline , ...
    numpanels2x_female.*numpanels1x_evangelical , numpanels2x_female.*numpanels1x_protestant , ...
    numpanels2x_female.*numpanels1x_ba_public , numpanels2x_female.*numpanels1x_elev , ...
    numpanels2x_female.*numpanels1x_nonwhite , ...
    numpanels2x_ba_public.*numpanels1x_jewish , numpanels2x_ba_public.*numpanels1x_catholic , ...
    numpanels2x_ba_public.*numpanels1x_noreligion , numpanels2x_ba_public.*numpanels1x_instate_ba , ...
    numpanels2x_ba_public.*numpanels1x_mainline , ...
    numpanels2x_ba_public.*numpanels1x_evangelical , numpanels2x_ba_public.*numpanels1x_protestant , ...
    numpanels2x_ba_public.*numpanels1x_elev , ...
    numpanels2x_ba_public.*numpanels1x_nonwhite , ...
    numpanels1x_instate_ba.*numpanels1x_jewish , numpanels1x_instate_ba.*numpanels1x_catholic , ...
    numpanels1x_instate_ba.*numpanels1x_noreligion , numpanels1x_instate_ba.*numpanels1x_instate_ba , ...
    numpanels1x_instate_ba.*numpanels1x_mainline , ...
    numpanels1x_instate_ba.*numpanels1x_evangelical , numpanels1x_instate_ba.*numpanels1x_protestant , ...
    numpanels1x_instate_ba.*numpanels1x_elev , ...
    numpanels1x_instate_ba.*numpanels1x_nonwhite , ...
    numpanels1x_elev.*numpanels1x_jewish , numpanels1x_elev.*numpanels1x_catholic , ...
    numpanels1x_elev.*numpanels1x_noreligion , ...
    numpanels1x_elev.*numpanels1x_mainline , ...
    numpanels1x_elev.*numpanels1x_evangelical , numpanels1x_elev.*numpanels1x_protestant , ...
    numpanels1x_elev.*numpanels1x_nonwhite , ...
    numpanels1x_protestant.*numpanels1x_jewish , numpanels1x_protestant.*numpanels1x_catholic , ...
    numpanels1x_protestant.*numpanels1x_noreligion , ...
    numpanels1x_protestant.*numpanels1x_mainline , ...
    numpanels1x_protestant.*numpanels1x_evangelical , numpanels1x_protestant.*numpanels1x_protestant , ...
    numpanels1x_protestant.*numpanels1x_nonwhite , ...
    numpanels1x_mainline.*numpanels1x_jewish , numpanels1x_mainline.*numpanels1x_catholic , ...
    numpanels1x_mainline.*numpanels1x_noreligion , numpanels1x_mainline.*numpanels1x_evangelical , ...
    numpanels1x_mainline.*numpanels1x_mainline , ...
    numpanels1x_mainline.*numpanels1x_nonwhite ];
% Candidate instruments' names, namez
namez = {'Z1xNR', 'Z1xJDP', 'Z1xD', 'Z2xD', 'Z3xD', ...
    'Z1xDSq', 'Z1xDCu', ...
    'Z1xF', 'Z2xF', 'Z3xF', ...
    'Z1xNW', 'Z2xNW', 'Z3xNW', ...
    'Z1xB', 'Z2xB', 'Z3xB', ...
    'Z1xJ', 'Z2xJ', 'Z3xJ', ...
    'Z1xCAT', 'Z2xCAT', 'Z3xCAT', ...
    'Z2xNR', 'Z3xNR', ...
    'Z1xIBA', 'Z2xIBA', 'Z3xIBA', ...
    'Z1xBAP', 'Z2xBAP', 'Z3xBAP', ...
    'Z2xJDP', 'Z3xJDP', ...
    'Z1xJDPSq', 'Z1xJDPCu', ...
    'Z1xE', 'Z2xE', 'Z3xE', ...
    'Z1xESq', 'Z1xECu', ...
    'Z1xFB', 'Z2xFB', 'Z3xFB', ...
    'Z1xFNR', 'Z2xFNR', 'Z3xFNR', ...
    'Z1xBNR', 'Z2xBNR', 'Z3xBNR', ...
    'Z1xFJDP', 'Z2xFJDP', 'Z3xFJDP', ...
    'Z1xBJDP', 'Z2xBJDP', 'Z3xBJDP', ...
    'Z1xNRJDP', 'Z2xNRJDP', 'Z3xNRJDP', ...
    'Z1xM', 'Z2xM', 'Z3xM', ...
    'Z1xEV', 'Z2xEV', 'Z3xEV', ...
    'Z1xPRO', 'Z2xPRO', 'Z3xPRO', ...
    'Z1xDC1xF', 'Z1xDC1xB', ...
    'Z1xDC1xJ', 'Z1xDC1xCAT', ...
    'Z1xDC1xNR', 'Z1xDC1xBAP', ...
    'Z1xDC1xJDP', 'Z1xDC1xM', ...
    'Z1xDC1xEV', 'Z1xDC1xPRO', ...
    'Z1xDC1xBAP', 'Z1xDC1xE', 'Z1xDC1xNW', ... 
    'Z1xFC1xJ', 'Z1xFC1xCAT', ...
    'Z1xFC1xNR', 'Z1xFC1xBAP', ...
    'Z1xFC1xJDP', 'Z1xFC1xM', ...
    'Z1xFC1xEV', 'Z1xFC1xPRO', ...
    'Z1xFC1xBAP', 'Z1xFC1xE', 'Z1xFC1xNW', ... 
    'Z1xJDPC1xJ', 'Z1xJDPC1xCAT', ...
    'Z1xJDPC1xNR', 'Z1xJDPC1xIBA', ...
    'Z1xJDPC1xM', ...
    'Z1xJDPC1xEV', 'Z1xJDPC1xPRO', ...
    'Z1xJDPC1xBAP', 'Z1xJDPC1xE', ...
    'Z1xJDPC1xNW', ...
    'Z1xBAPC1xJ', 'Z1xBAPC1xCAT', ...
    'Z1xBAPC1xNR', 'Z1xBAPC1xIBA', ...
    'Z1xBAPC1xM', ...
    'Z1xBAPC1xEV', 'Z1xBAPC1xPRO', ...
    'Z1xBAPC1xE', ...
    'Z1xBAPC1xNW', ...
    'Z2xJDPC1xJ', 'Z2xJDPC1xCAT', ...
    'Z2xJDPC1xNR', 'Z2xJDPC1xIBA', ...
    'Z2xJDPC1xM', ...
    'Z2xJDPC1xEV', 'Z2xJDPC1xPRO', ...
    'Z2xJDPC1xBAP', 'Z2xJDPC1xE', ...
    'Z2xJDPC1xNW', ...
    'Z3xJDPC1xJ', 'Z3xJDPC1xCAT', ...
    'Z3xJDPC1xNR', 'Z3xJDPC1xIBA', ...
    'Z3xJDPC1xM', ...
    'Z3xJDPC1xEV', 'Z3xJDPC1xPRO', ...
    'Z3xJDPC1xBAP', 'Z3xJDPC1xE', ...
    'Z3xJDPC1xNW', ...
    'Z2xFC1xJ', 'Z2xFC1xCAT', ...
    'Z2xFC1xNR', 'Z2xFC1xBAP', ...
    'Z2xFC1xJDP', 'Z2xFC1xM', ...
    'Z2xFC1xEV', 'Z2xFC1xPRO', ...
    'Z2xFC1xBAP', 'Z2xFC1xE', 'Z2xFC1xNW', ... 
    'Z2xBAPC1xJ', 'Z2xBAPC1xCAT', ...
    'Z2xBAPC1xNR', 'Z2xBAPC1xIBA', ...
    'Z2xBAPC1xM', ...
    'Z2xBAPC1xEV', 'Z2xBAPC1xPRO', ...
    'Z2xBAPC1xE', ...
    'Z2xBAPC1xNW', ...
    'Z1xIBAC1xJ', 'Z1xIBAC1xCAT', ...
    'Z1xIBAC1xNR', 'Z1xIBAC1xIBA', ...
    'Z1xIBAC1xM', ...
    'Z1xIBAC1xEV', 'Z1xIBAC1xPRO', ...
    'Z1xIBAC1xE', ...
    'Z1xIBAC1xNW', ...    
    'Z1xEC1xJ', 'Z1xEC1xCAT', ...
    'Z1xEC1xNR', 'Z1xEC1xM', ...
    'Z1xEC1xEV', 'Z1xEC1xPRO', ...
    'Z1xEC1xNW', ...
    'Z1xPROC1xJ', 'Z1xPROC1xCAT', ...
    'Z1xPROC1xNR', 'Z1xPROC1xM', ...
    'Z1xPROC1xEV', 'Z1xPROC1xPRO', ...
    'Z1xPROC1xNW', ...
    'Z1xMC1xJ', 'Z1xMC1xCAT', ...
    'Z1xMC1xNR', ...
    'Z1xMC1xEV', 'Z1xMC1xM', ...
    'Z1xMC1xNW' }';

% Probability controls, xP:
xP = 1*[F_prob_1x_dem, F_prob_2x_dem, F_prob_3x_dem, ...
    F_prob_1x_female, F_prob_2x_female, F_prob_3x_female, ...
    F_prob_1x_nonwhite, F_prob_2x_nonwhite, F_prob_3x_nonwhite, ...
    F_prob_1x_black, F_prob_2x_black, F_prob_3x_black, ...
    F_prob_1x_jewish, F_prob_2x_jewish, F_prob_3x_jewish, ...
    F_prob_1x_catholic, F_prob_2x_catholic, F_prob_3x_catholic, ...
    F_prob_1x_noreligion, F_prob_2x_noreligion, F_prob_3x_noreligion, ...
    F_prob_1x_instate_ba, F_prob_2x_instate_ba, F_prob_3x_instate_ba, ...
    F_prob_1x_ba_public, F_prob_2x_ba_public, F_prob_3x_ba_public, ...
    F_prob_1x_jd_public, F_prob_2x_jd_public, F_prob_3x_jd_public, ...
    F_prob_1x_elev, F_prob_2x_elev, F_prob_3x_elev, ...
    F_prob_1x_female_black, F_prob_2x_female_black, F_prob_3x_female_black, ...
    F_prob_1x_female_noreligion, F_prob_2x_female_noreligion, F_prob_3x_female_noreligion, ...
    F_prob_1x_black_noreligion, F_prob_2x_black_noreligion, F_prob_3x_black_noreligion, ...
    F_prob_1x_female_jd_public, F_prob_2x_female_jd_public, F_prob_3x_female_jd_public, ...
    F_prob_1x_black_jd_public, F_prob_2x_black_jd_public, F_prob_3x_black_jd_public, ...
    F_prob_1x_noreligion_jd_public, F_prob_2x_noreligion_jd_public, F_prob_3x_noreligion_jd_public, ...
    F_prob_1x_mainline, F_prob_2x_mainline, F_prob_3x_mainline, ...
    F_prob_1x_evangelical, F_prob_2x_evangelical, F_prob_3x_evangelical, ...
    F_prob_1x_protestant, F_prob_2x_protestant, F_prob_3x_protestant ]; 

% # Democrats instrument
IND_dem = zeros(size(z,2),1);
IND_dem(3) = 1;
IND_dem = logical(IND_dem);

%% Preliminary Data Cleaning
% Partial out x's that need to show up in both stages
I = find(mean(z) >= .05);
z = z(:,I);
IND_dem = IND_dem(I);
namez = namez(I);
y = logpriceindex;
I = find(mean(xP) >= .05);
xP = xP(:,I); %#ok<FNDSB>
x = [x xP];

% Collapse to circuit-year level
x = inv(Dt'*Dt)*(Dt'*x);
y = inv(Dt'*Dt)*(Dt'*y);
d = inv(Dt'*Dt)*(Dt'*d);
z = inv(Dt'*Dt)*(Dt'*z);

xxinv = inv(x'*x);
My = full(y - x*xxinv*(x'*y)); %#ok<*MINV>
Md = full(d - x*xxinv*(x'*d));
Mz = full(z - x*xxinv*(x'*z));

I = find(std(Mz) > 1e-6);
Mz = Mz(:,I);
IND_dem = IND_dem(I);
namez = namez(I);

[I,J] = find(abs(tril(corr(Mz)-eye(size(Mz,2)))) > .99);
drop = unique(I);
Mz(:,drop) = [];
IND_dem(drop) = [];
namez(drop) = [];

n = size(Mz,1);
p = size(Mz,2);
kx = size(x,2);

%% Write relevant data to be used in IV analysis to take in to Stata
NAMES = ['CSIndex';'NumProCase';namez];
xlswrite('CSExampleData.xlsx',NAMES',1,'A1');
DATA = [My Md Mz];
xlswrite('CSExampleData.xlsx',DATA,1,'A2');
xlswrite('CSExampleData.xlsx',{'nControl'},1,'ET1');
xlswrite('CSExampleData.xlsx',kx,1,'ET2');
